/* This stylesheet is generated, DO NOT EDIT */
/* Copyright 2009, 2015 Red Hat, Inc.
 *
 * Portions adapted from Mx's data/style/default.css
 *   Copyright 2009 Intel Corporation
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU Lesser General Public License,
 * version 2.1, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 */
.svm-menu {
  /**
   * Common
   */
  /**
   * SVM integrated in status menu dropdown
   */
  /**
   * Stand-alone menu via extra-button
   */
}
.svm-menu .svm-slider-details {
  font-size: 0.9em;
  font-weight: 400;
}
.svm-menu .popup-menu-item:ltr {
  margin-left: 0;
  padding-left: 0;
}
.svm-menu .popup-menu-item:rtl {
  margin-right: 0;
  padding-right: 0;
}
.svm-menu .popup-sub-menu .popup-menu-item:ltr {
  padding-right: 14px;
}
.svm-menu .popup-sub-menu .popup-menu-item:rtl {
  padding-left: 14px;
}
.svm-menu .popup-sub-menu .popup-menu-item > .svm-menu-item-container {
  padding-left: 0;
  padding-right: 0;
  margin-left: 0;
  margin-right: 0;
}
.svm-menu .popup-sub-menu .popup-menu-item > .svm-menu-item-container > .svm-container-line {
  /* somewhere along all these paddings there's too much added
   * here, offsetting sliders in submenus by a few pixels
   */
}
.svm-menu .popup-sub-menu .popup-menu-item > .svm-menu-item-container > .svm-container-line:ltr {
  padding-right: 12px;
}
.svm-menu .popup-sub-menu .popup-menu-item > .svm-menu-item-container > .svm-container-line:rtl {
  padding-left: 12px;
}
.svm-menu > .popup-menu-item > .svm-menu-item-container {
  padding-left: 0;
  padding-right: 0;
  margin-left: 0;
  margin-right: 0;
}
.svm-menu > .popup-menu-item > .svm-menu-item-container > .svm-container-line {
  padding-left: 0;
  padding-right: 0;
  margin-left: 0;
  margin-right: 0;
}
.svm-menu.svm-integrated-menu .popup-sub-menu .popup-menu-item > .svm-menu-item-container > .svm-container-line.line-1 .popup-menu-ornament:ltr {
  margin-left: 0;
  padding-left: 0;
}
.svm-menu.svm-integrated-menu .popup-sub-menu .popup-menu-item > .svm-menu-item-container > .svm-container-line.line-1 .popup-menu-ornament:rtl {
  margin-right: 0;
  padding-right: 0;
}
.svm-menu.svm-integrated-menu > .popup-menu-item > .svm-menu-item-container .svm-master-slider-line > .popup-menu-ornament,
.svm-menu.svm-integrated-menu > .popup-menu-item > .svm-menu-item-container .svm-events-slider-line > .popup-menu-ornament {
  /** @copypaste from gnome-shell .aggregate-menu .popup-sub-menu .popup-menu-item > :first-child **/
}
.svm-menu.svm-integrated-menu > .popup-menu-item > .svm-menu-item-container .svm-master-slider-line > .popup-menu-ornament:ltr,
.svm-menu.svm-integrated-menu > .popup-menu-item > .svm-menu-item-container .svm-events-slider-line > .popup-menu-ornament:ltr {
  padding-left: 14px;
  margin-left: 1.09em;
}
.svm-menu.svm-integrated-menu > .popup-menu-item > .svm-menu-item-container .svm-master-slider-line > .popup-menu-ornament:rtl,
.svm-menu.svm-integrated-menu > .popup-menu-item > .svm-menu-item-container .svm-events-slider-line > .popup-menu-ornament:rtl {
  padding-right: 14px;
  margin-right: 1.09em;
}
.svm-menu.svm-standalone-menu {
  width: 30em;
}
